''*****************************************************
''* This is a program that shows how to send and      *
''* receive TCP data as a TCP client. 
''* Written by: Shane Avery                           *
''***************************************************** 

CON

        _clkmode = xtal1 + pll8x
        _xinfreq = 10_000_000 + 0000

        'Constants for UDP for TCP port
        UDP = 0
        TCP = 1

        'Define the port number
        PORT = 120

VAR
        byte  dip[4]
        byte  subnet[4]
        byte  ip[4]
        byte  gateway[4]
        byte  mac[8]
        byte  buffer[256]                   

OBJ

        term    : "tv_terminal_010.spin"     ' instantiate the terminal object
        eth     : "W5100_drv_011.spin"       ' instantiate the W5100 driver

PUB begin | size

  'Start the tv terminal.
  term.start

  'Start the W5100 driver.
  eth.start

  'Display a title string and indicate we are
  'waiting for the ethernet to come up.
  term.out($02)
  term.pstring(string("HydraEtherX TCP Client Test",13,13))
  term.out($01)
  term.pstring(string("Waiting for Eth",13))
  
  'Wait for awhile for the ethernet to come up.
  waitcnt(cnt + $17d78400)

  'Indicate that we are done waiting.
  term.pstring(string("Done waiting",13,13))

  'Fill out the arrays needed for intialization.

  'Destination IP is IP we are sending to
  dip[0] := 192
  dip[1] := 168
  dip[2] := 1
  dip[3] := 2

  'Fill out the rest of the arrays needed for intialization.
  'W5100 IP address.
  ip[0] := 192
  ip[1] := 168
  ip[2] := 1
  ip[3] := 100

  'Gateway.
  gateway[0] := 192
  gateway[1] := 168
  gateway[2] := 1
  gateway[3] := 1

  'Subnet.
  subnet[0] := 255
  subnet[1] := 255
  subnet[2] := 255
  subnet[3] := 0

  'W5100 MAC address.
  mac[0] := $00
  mac[1] := $70
  mac[2] := $6e
  mac[3] := $69
  mac[4] := $73
  mac[5] := $0a

  'Initialize the W5100.
  eth.init(@gateway, @subnet, @ip, @mac)
  
  'Open the socket for TCP communication.
  eth.open(TCP, PORT, PORT, @dip)

  'Display message.
  term.pstring(string("Connecting...",13))
  
  'Connect to the PC
  eth.connect
   
  'Wait for connection to be established.
  repeat while eth.con_est == false
   
  'Display connected message.
  term.pstring(string("Connected to PC",13,13))

  'Read the data from the W5100 buffer.
  term.pstring(string("Waiting for data from PC",13))
  size := eth.RX(@buffer, 256, true)

  'Display how many bytes we received.
  term.pstring(string("Got "))
  term.dec(size)
  term.pstring(string(" bytes from the W5100", 13))

  'Display the payload data.
  term.pstring(string("Data: "))
  term.pstring(@buffer[0])
  term.out(13)

  'Send data to PC.
  term.pstring(string("Sending data to PC",13))  
  eth.TX(@hello, 21)
  term.pstring(string("Sent data",13,13))

  term.out($02)
  term.pstring(string("DONE"))

  repeat

DAT

hello   byte  "Hello from the Hydra",0