''*****************************************************
''* This is a program that shows how to send and      *
''* receive UDP data. 
''* Written by: Shane Avery                           *
''*****************************************************            

CON

        _clkmode = xtal1 + pll8x
        _xinfreq = 10_000_000 + 0000

        'Constants for UDP for TCP port
        UDP = 0
        TCP = 1

        'Define the port number
        PORT = 120

VAR
        byte  dip[4]
        byte  subnet[4]
        byte  ip[4]
        byte  gateway[4]
        byte  mac[8]
        byte  buffer[256]
        
OBJ

        term    : "tv_terminal_010.spin"     ' instantiate the terminal object
        eth     : "W5100_drv_011.spin"       ' instantiate the W5100 driver

PUB begin | size, i, srcport, length

  'Start the tv terminal.
  term.start

  'Start the W5100 driver.
  eth.start

  'Display a title string and indicate we are
  'waiting for the ethernet to come up.
  term.out($02)
  term.pstring(string("HydraEtherX UDP Test",13,13))
  term.out($01)
  term.pstring(string("Waiting for Eth",13))
  
  'Wait for awhile for the ethernet to come up.
  waitcnt(cnt + $17d78400)

  'Indicate that we are done waiting
  term.pstring(string("Done waiting",13,13))

  'Fill out the arrays needed for intialization.
  
  'Destination IP is the address of the computer we
  'will communicate with.
  dip[0] := 192
  dip[1] := 168
  dip[2] := 1
  dip[3] := 2

  'W5100 IP address.
  ip[0] := 192
  ip[1] := 168
  ip[2] := 1
  ip[3] := 100

  'Gateway.
  gateway[0] := 192
  gateway[1] := 168
  gateway[2] := 1
  gateway[3] := 1

  'Subnet.
  subnet[0] := 255
  subnet[1] := 255
  subnet[2] := 255
  subnet[3] := 0

  'W5100 MAC address.
  mac[0] := $00
  mac[1] := $70
  mac[2] := $6e
  mac[3] := $69
  mac[4] := $73
  mac[5] := $0a

  'Initialize the W5100.
  eth.init(@gateway, @subnet, @ip, @mac)
  
  'Open the socket for UDP communication.
  eth.open(UDP, PORT, PORT, @dip)

  'Transmit some data to the PC.
  term.pstring(string("Sending data to PC",13))  
  eth.TX(@hello, 21)
  term.pstring(string("Sent data",13,13))
  
  'Receive data from the PC.
  term.pstring(string("Waiting for data from PC",13))
  size := eth.RX(@buffer, 256, true)

  'Display how many bytes we received.
  term.pstring(string("Got "))
  term.dec(size)
  term.pstring(string(" bytes from the W5100", 13))

  'Display the RX header info.
  'Display the IP we got the address from
  term.pstring(string("Got data from IP: "))
  term.dec(buffer[0])
  term.out($2e)
  term.dec(buffer[1])
  term.out($2e)
  term.dec(buffer[2])
  term.out($2e)
  term.dec(buffer[3])
  term.out(13)

  'Display the source port
  term.pstring(string("Source Port: "))
  srcport := (buffer[4] << 8) + buffer[5]
  term.dec(srcport)
  term.out(13)

  'Display the data payload size 
  term.pstring(string("Payload size: "))
  length := (buffer[6] << 8) + buffer[7]
  term.dec(length)
  term.out(13)

  'Display the payload data
  term.pstring(string("Data: "))
  term.pstring(@buffer[8])
  term.out(13)
   

  term.out($02)
  term.pstring(string("DONE"))

  repeat
DAT
hello   byte  "Hello from the Hydra",0
