''*****************************************************
''* Squish Kevin game.
''* Written by: Shane Avery
''*****************************************************

CON 
  _clkmode = xtal1 + pll8x
  _xinfreq = 10_000_000 + 0000
  _stack = ($3000 + 64) >> 2 

  ' graphics driver and screen constants
  PARAMCOUNT = 14
  ONSCREEN_BUFFER = $5000

  ' size of the graphics tile map
  X_TILES = 16
  Y_TILES = 12

  SCREEN_WIDTH = 256
  SCREEN_HEIGHT = 192

  'Color constants 
  COL_Black     = %0000_0010
  COL_Blue      = %0001_1_100
  COL_Green     = %0101_1_100
  COL_Red       = %1010_1_100
  COL_White     = %0000_0111

  COLOR_0 = (COL_Black << 0)
  COLOR_1 = (COL_Blue << 8)
  COLOR_2 = (COL_Red << 16)
  COLOR_3 = (COL_White << 24)

  ' NES bit encodings for NES gamepad 0
  NES0_RIGHT  = %00000000_00000001
  NES0_LEFT   = %00000000_00000010
  NES0_DOWN   = %00000000_00000100
  NES0_UP     = %00000000_00001000
  NES0_START  = %00000000_00010000
  NES0_SELECT = %00000000_00100000
  NES0_B      = %00000000_01000000
  NES0_A      = %00000000_10000000

VAR

  long  tv_status     '0/1/2 = off/visible/invisible           read-only
  long  tv_enable     '0/? = off/on                            write-only
  long  tv_pins       '%ppmmm = pins                           write-only
  long  tv_mode       '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen     'pointer to screen (words)               write-only
  long  tv_colors     'pointer to colors (longs)               write-only               
  long  tv_hc         'horizontal cells                        write-only
  long  tv_vc         'vertical cells                          write-only
  long  tv_hx         'horizontal cell expansion               write-only
  long  tv_vx         'vertical cell expansion                 write-only
  long  tv_ho         'horizontal offset                       write-only
  long  tv_vo         'vertical offset                         write-only
  long  tv_broadcast  'broadcast frequency (Hz)                write-only
  long  tv_auralcog   'aural fm cog                            write-only

  word  screen[x_tiles * y_tiles]
  long  colors[64]

  byte  dip[4]        'destination ip. the ip address we wish to commune with          
  byte  subnet[4]     'subnet
  byte  ip[4]         'our ip address
  byte  gateway[4]    'gateway ip address
  byte  mac[8]        'our mac address

  long  padstate
  long  sx, sy, sa    'shane coordinates and attack status
  long  kx, ky, ka    'kevin coordinates and attack status
  'long  sxtemp, sytemp, satemp    
  'long  kxtemp, kytemp, katemp      
  long  unitfound     'variable to indicate that we are talking to the other Hydra

  long  cog_stack[160] 'stack space for the new cogs

  long  rxchange
  long  statchanged
  byte  rxchangelock  'lock to change the states of Kevin's variables
  byte  txchangelock          

OBJ

  tv    : "tv_drv_010.spin"
  gr    : "graphics_drv_010"
  pad   : "gamepad_drv_001"
  eth   : "W5100_drv_011"

DAT                           
tvparams                long    0               'status
                        long    1               'enable
                        long    %011_0000       'pins
                        long    %0000           'mode
                        long    0               'screen
                        long    0               'colors
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    10              'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    60_000_000'_xinfreq<<4  'broadcast
                        long    0               'auralcog

PUB start | i, dx, dy, padpressed, sscore, kscore, sxtemp, sytemp, satemp, kxtemp, kytemp, katemp

  '///////////////////////////////////////////////////////////
  'Begin the initialization and display Splash screen
  '///////////////////////////////////////////////////////////
  
  'start gamepad
  pad.start
  
  'start the W5100 driver
  eth.start

  'start tv
  longmove(@tv_status, @tvparams, paramcount)
  tv_screen := @screen
  tv_colors := @colors
  tv.start(@tv_status)

  'init colors, each tile has same 4 colors
  repeat i from 0 to 64
    colors[i] := COLOR_3 | COLOR_2 | COLOR_1 | COLOR_0

  'init tile screen
  repeat dx from 0 to tv_hc - 1
    repeat dy from 0 to tv_vc - 1
      screen[dy * tv_hc + dx] := ONSCREEN_BUFFER >> 6 + dy + dx * tv_vc + ((dy&$3f)<<10)

  'start and setup the graphics driver
  gr.start
  gr.setup(X_TILES, Y_TILES, 0, 0, ONSCREEN_BUFFER)

  'init the game control varaibles
  unitfound := 0
  rxchange := 0
  statchanged := 0
  sx := 0
  sy := 0
  sa := 0
  kx := 0
  ky := 0
  ka := 0
  sxtemp := 0
  sytemp := 0
  satemp := 0
  kxtemp := 0
  kytemp := 0
  katemp := 0
  sscore := 0
  kscore := 0

  'display the splash screen for the game
  gr.clear
  gr.colorwidth(2,2)
  gr.textmode(2,2,6,0)
  gr.text(50,150,@title)
  waitcnt(cnt + $17d78400)      'we will wait a few seconds for the ethernet to come up

  'Initialize the ethernet device
  InitEther

  'Create a new lock for Kevin's data
  rxchangelock := locknew
  txchangelock := locknew

  'Start the routine which will send and receive data via
  'the ethernet in a new cog.
  cognew(TxRxData, @cog_stack[0])

  'Now that the ethernet is up the user can be prompted
  'to push start to begin the same
  gr.colorwidth(1,0)
  gr.textmode(1,1,6,0)
  gr.text(90,130,@pushst)

  'While waiting for the start button look to see if we are
  'getting data from the other Hydra and if so indicate that
  repeat while pad.read & NES0_START == 0
    if(unitfound == 1)
      gr.colorwidth(3,0)
      gr.text(70,40,@unitfnd)

  'Start the status monitor
  cognew(StatusChange, @cog_stack[128])

  '//////////////////////////////////////////////////////////
  'Begin the game loop
  '////////////////////////////////////////////////////////// 
  'Drawing us on the playing field.
  gr.clear
  gr.colorwidth(1,10)
  gr.plot(50+sx, 100+sy)
  gr.colorwidth(2,10)
  gr.plot(210-kx, 100+ky)
  
  'The main game loop
  repeat

    'Wait for our status to change or for Kevin's status to change
    repeat until (statchanged == 1 or rxchange == 1)

    'If our status has changed then update the display and the
    'global status data.
    if(statchanged == 1)

      'Clear the variable that got us here
      statchanged := 0
       
      'Determine our movements
      if(padstate & NES0_UP)
        if(sy < 80) 'Check to see if we have hit the boundary               
          sytemp += 10
      if(padstate & NES0_DOWN)
        if(sy > -90)
          sytemp -= 10
      if(padstate & NES0_RIGHT)
        if(sx < 200)
          sxtemp += 10
      if(padstate & NES0_LEFT)
        if(sx > -40)
          sxtemp -= 10
       
      'If the player hits a button then they attack.
      'Once the attack is started they must wait a
      'small amount of time before they can do anything else.
      if(padstate & (NES0_A | NES0_B))
        if(((210-kxtemp) == (50+sxtemp)) and kytemp == sytemp)
          satemp := 2
        else
          satemp := 1
      else
        satemp := 0
       
      'Grab the lock to write our new status data to
      'the global variables
      repeat while(lockset(txchangelock) == true)
      sy := sytemp
      sx := sxtemp
      sa := satemp
      lockclr(txchangelock)

    'If Kevin's status has changed then we need
    'to get his data
    if(rxchange == 1)
    
      'Grab lock to get Kevin's new status data
      repeat while(lockset(rxchangelock) == true)
      kxtemp := kx
      kytemp := ky
      katemp := ka
      rxchange := 0
      lockclr(rxchangelock)

    'Draw new states  
    gr.clear

    'Determine if Kevin is attacking. If so we need to
    'modify his massive pixel to be less massive
    if(katemp == 1 or katemp == 2)
      gr.colorwidth(2,6)
    else
      gr.colorwidth(2,10)
    gr.plot(210-kxtemp, 100+kytemp)

    'Determine if we are attacking. If so we need to
    'modify his massive pixel to be less massive.
    if(satemp == 1 or satemp == 2)
      gr.colorwidth(1,6)
    else
      gr.colorwidth(1,10)
    gr.plot(50+sxtemp,100+sytemp)
    
    'See if anyone won. First check to see if we are on
    'top of each other. Then determine who is attacking.
    if(((210-kxtemp) == (50+sxtemp)) and kytemp == sytemp)
      if(satemp == 2 or katemp == 2)
      
        waitcnt(cnt + 10_000_000)
        
        'If Kevin's status has changed then we need
        'to get his data
        if(rxchange == 1)
         
          'Grab lock to get Kevin's new status data
          repeat while(lockset(rxchangelock) == true)
          kxtemp := kx
          kytemp := ky
          katemp := ka
          rxchange := 0
          lockclr(rxchangelock)
        
        gr.colorwidth(3,2)
        gr.textmode(2,2,6,0)

        if(satemp == 2 and katemp == 2)
          gr.text(90,130,@tie)

        'If we win display victory and inc our counter.
        elseif(satemp == 2)
          gr.text(70,130,@vic)
          sscore += 1
          score[0] += 1

        'If we loose display defeat and inc Kevin's counter.
        elseif(katemp == 2)
          gr.text(70,130,@def)
          kscore += 1
          score[2] += 1

        'Display the current score
        gr.text(100,100,@score)

        'Wait for 2 seconds for the players to see the result.
        waitcnt(cnt + 160_000_000)

        'Re-init the game variables.
        sx := 0
        sy := 0
        sa := 0
        kx := 0
        ky := 0
        ka := 0
        sxtemp := 0
        sytemp := 0
        satemp := 0
        kxtemp := 0
        kytemp := 0
        katemp := 0
        statchanged := 1
        padstate := 0

        'If one of us has reached 5 then the game is over
        'so reset the scores back to zero
        if(kscore == 5 or sscore == 5)
          sscore := 0
          kscore := 0
          score[0] := "0"
          score[2] := "0"
      
              
      
PUB TxRxData | data, size, kxlocal, kylocal, kalocal, buf[3], count

  'This routine will be in a separate cog constantly running.
  'We will read data as it comes in from the other Hydra and
  'transmit data 16 times a second.

  'Begin by sampling the global counter
  count := cnt
  
  repeat
    'Try to read data. This is a non-blocking call. Thus,
    'if there is no data available then it will return
    'right away with a size of zero
    size := eth.RX(@data, 4, false)  'Read the first 4 bytes of UDP header                     

    'If the size was greater than zero then we have data.
    'We are making a dangerous assumption here that the
    'only data to have arrived is the data from the other
    'Hydra and we KNOW that it will send 3 words of data.
    'Note here however, that we did read 5 words (including
    'the read above). This is because when the W5100 reads
    'UDP data the UDP header is included in the data. We have
    'to read from the W5100 to get at the data we want. Thus,
    'we will read it to a dummy variable, data, and then junk it.
    if(size > 0)
      eth.RX(@data, 4, false)   'Read the last 4 bytes of the UDP header
      eth.RX(@kxlocal, 4, false)     'Now read the actual data sent from the
      eth.RX(@kylocal, 4, false)     '  the other hydra to the variables that
      eth.RX(@kalocal, 4, false)     '  they correspond to

      'Change the unitfound variable to 1 to indicate that we are
      'talking to the other Hydra
      unitfound := 1

    'Check to see if there has been any change to the status of
    'the other Hydra. If so then we will set change to 1 so that
    'the main game loop can update the changes in the display
    if(kxlocal <> kx or kylocal <> ky or kalocal <> ka)
      repeat while(lockset(rxchangelock) == true)
      rxchange := 1
      kx := kxlocal
      ky := kylocal
      ka := kalocal
      lockclr(rxchangelock)

    'Check the global counter to see if we should send data now.
    'The first if statement is complicated but basically it is trying
    'to handle the case of the global counter overflowing.
    if( ((count + 5_000_000) > count) or (((count + 5_000_000) < count) and (cnt < $7FFFFFFF))  )
      if (count + 5_000_000 < cnt)

        'Resample the global counter for the next time.
        count := cnt

        'Grab the lock to read our status data. This keeps the
        'main game loop frlom writing values while we are
        'reading them. Then we will store the status data
        'to a small local buffer.
        repeat while(lockset(txchangelock) == true)
        buf[0] := sx  'Store our data to a three array buffer                
        buf[1] := sy
        buf[2] := sa
        lockclr(txchangelock)

        'Send the status of us to the other Hydra by writing
        'the small buffer that we stored the status data in.
        eth.TX(@buf, 12)

PUB StatusChange

  'This routine will run in another cog and monitor our status.
  'If we are in attack status then this routine will wait for
  'about 1/2 second as a penalty.
  repeat 
  
    'Wait for the player to stop pushing on the pad
    repeat until (pad.read & $ff == 0)

      'If we are in attack mode then we will wait for
      '1/2 second and then clear the global attack
      'variable. Since we will clear this variable we
      'need to grab the lock so that the TxRx routine
      'won't access this memory while we are writing it.
      if(sa == 1 or sa == 2)
        waitcnt(cnt+40_000_000)
        repeat while(lockset(txchangelock) == true)
        sa := 0
        lockclr(txchangelock)

        'Sample the game pad
        padstate := pad.read
        
        'Set this variable to tell the main loop that
        'we are no longer attacking
        statchanged := 1

    'Wait for the player to push on the pad
    repeat until (pad.read & $ff <> 0)
  
      if(sa == 1 or sa == 2)
        waitcnt(cnt+40_000_000)
        repeat while(lockset(txchangelock) == true)
        sa := 0
        lockclr(txchangelock)

        'Sample the game pad
        padstate := pad.read

        'Set this variable to tell the main loop that
        'we are no longer attacking
        statchanged := 1

    'Sample the game pad
    padstate := pad.read

    'Set this variable to tell the main lopp that we
    'have a change in the pad status
    statchanged := 1
    
  
    
CON

  'Constants for UDP for TCP port
  UDP = 0
  TCP = 1
   
  'Define the port number
  PORT = 200

  'Define constant for the Ethernet initialization
  'Destination IP. The address we want to commune with
  DIP_0 = 192
  DIP_1 = 168
  DIP_2 = 1
  DIP_3 = 101

  'IP. This is our IP address
  IP_0  = 192
  IP_1  = 168
  IP_2  = 1
  IP_3  = 100

  'Gateway. This is the gateway ip address
  GWAY_0 = 192
  GWAY_1 = 168
  GWAY_2 = 1
  GWAY_3 = 1

  'Subnet
  SUB_0 = 255
  SUB_1 = 255
  SUB_2 = 255
  SUB_3 = 0

  'MAC address. This is our MAC address.
  MAC_0 = $00
  MAC_1 = $70
  MAC_2 = $6e
  MAC_3 = $69
  MAC_4 = $73
  MAC_5 = $0a        

PUB InitEther

  'This function will intialize the W5100

  'Destination IP is the IP of the Ethernet card we want to talk to
  dip[0] := DIP_0
  dip[1] := DIP_1
  dip[2] := DIP_2
  dip[3] := DIP_3

  'fill out the rest of the arrays needed for intialization
  'This is our IP address
  ip[0] := IP_0
  ip[1] := IP_1
  ip[2] := IP_2
  ip[3] := IP_3

  'Gateway
  gateway[0] := GWAY_0
  gateway[1] := GWAY_1
  gateway[2] := GWAY_2
  gateway[3] := GWAY_3

  'Subnet
  subnet[0] := SUB_0
  subnet[1] := SUB_1
  subnet[2] := SUB_2
  subnet[3] := SUB_3

  'W5100 MAC address
  mac[0] := MAC_0
  mac[1] := MAC_1
  mac[2] := MAC_2
  mac[3] := MAC_3
  mac[4] := MAC_4
  mac[5] := MAC_5

  'initialize the W5100
  eth.init(@gateway, @subnet, @ip, @mac)
  
  'open the socket
  eth.open(UDP, PORT, PORT, @dip)    
    
  
DAT

title   byte  "Squish Kevin",0
pushst  byte  "Push Start",0
unitfnd byte  "Other Unit Found",0
vic     byte  "VICTORY!!",0
def     byte  "DEFEAT!!",0
tie     byte  "TIE!!",0
score   byte  "0-0",0      
  