''*****************************************************
''* Tic Tac Toe game
''* Written by: Shane Avery
''*****************************************************

CON

  _clkmode = xtal1 + pll8x
  _xinfreq = 10_000_000 + 0000
   
  _stack = ($3000 + 200) >> 2   'accomodate display memory and stack

  x_tiles = 16
  y_tiles = 12

  paramcount = 14
  display_base = $5000

  'Constants for UDP for TCP port
  UDP = 0
  TCP = 1
   
  'Define the port number
  PORT = 200

  'Gamepad defines
  NES_RIGHT  = %00000001
  NES_LEFT   = %00000010
  NES_DOWN   = %00000100
  NES_UP     = %00001000
  NES_START  = %00010000
  NES_SELECT = %00100000
  NES_B      = %01000000
  NES_A      = %10000000

  'Contants for game states
  move = 0
  wait = 1
  over = 2

VAR         

  long  tv_status     '0/1/2 = off/visible/invisible           read-only
  long  tv_enable     '0/? = off/on                            write-only
  long  tv_pins       '%ppmmm = pins                           write-only
  long  tv_mode       '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen     'pointer to screen (words)               write-only
  long  tv_colors     'pointer to colors (longs)               write-only               
  long  tv_hc         'horizontal cells                        write-only
  long  tv_vc         'vertical cells                          write-only
  long  tv_hx         'horizontal cell expansion               write-only
  long  tv_vx         'vertical cell expansion                 write-only
  long  tv_ho         'horizontal offset                       write-only
  long  tv_vo         'vertical offset                         write-only
  long  tv_broadcast  'broadcast frequency (Hz)                write-only
  long  tv_auralcog   'aural fm cog                            write-only

  word  screen[x_tiles * y_tiles]
  long  colors[64]

  byte  dip[4]
  long  SPIRW, add0, add1, dataout, datain
  long  socket_type

  byte  marker[16]
  byte  buffer[4]
  byte  subnet[4]
  byte  ip[4]
  byte  gateway[4]
  byte  mac[8]

OBJ

        tv      : "tv_drv_010.spin"
        gr      : "graphics_drv_010.spin"
        eth     : "W5100_drv_011.spin"
        gp      : "gamepad_drv_001.spin"

PUB begin | i, j, dx, dy, x, y, pad, xo, gameover, state, movefirst

  'start ethernet driver
  eth.start

  'start gamepad
  gp.start

  'start tv
  longmove(@tv_status, @tvparams, paramcount)
  tv_screen := @screen
  tv_colors := @colors
  tv.start(@tv_status)

  'init colors
  repeat i from 0 to 64
    colors[i] := $00001010 * (i+4) & $F + $2B060C02

  'init tile screen
  repeat dx from 0 to tv_hc - 1
    repeat dy from 0 to tv_vc - 1
      screen[dy * tv_hc + dx] := display_base >> 6 + dy + dx * tv_vc + ((dy & $3F) << 10)

  'start and setup graphics
  gr.start
  gr.setup(16, 12, 43, 11, display_base)

  'clear the display
  gr.clear

  'display the splash screen
  gr.color(2)
  gr.text(47, 130, @ttt_text)
  gr.text(50, 110, @clienttext)
  gr.text(50, 100, @servertext)

  'wait for the user to determine if we are a client or server
  repeat while gp.read & (NES_A | NES_B) == $00

  'read the value of the gamepad 
  pad := gp.read

  'fill in as much of the ethernet/IP values that we can 
  dip[0] := 192
  dip[1] := 168
  dip[2] := 1
  dip[3] := 100

  ip[0] := 192
  ip[1] := 168
  ip[2] := 1

  gateway[0] := 192
  gateway[1] := 168
  gateway[2] := 1
  gateway[3] := 1

  subnet[0] := 255
  subnet[1] := 255
  subnet[2] := 255
  subnet[3] := 0

  mac[0] := $00
  mac[1] := $70
  mac[2] := $6e
  mac[3] := $69
  mac[4] := $73
  
  'if server then we'll have a different IP and MAC then if we are a client
  if(gp.read & NES_A)
    ip[3] := 100
    mac[5] := $0a

  'else if client fill in the different values
  else
    ip[3] := 101
    mac[5] := $0b

  'start the ethernet driver
  eth.init(@gateway, @subnet, @ip, @mac)
  
  'open the socket
  eth.open(TCP, PORT, PORT, @dip)

  'clear the screen
  gr.clear
  
  'if server init xo so we start as x and display a
  'listening text prompt until we get a connection
  if(gp.read & NES_A)
    xo := 0
    movefirst := true
    state := move
    gr.text(50, 90, @listentext)

    'listen for a connection
    eth.listen

  'if server init xo so we start as o and display a
  'connect text prompt until we get a connection
  else
    xo := 1
    movefirst := false
    state := wait
    gr.text(50, 90, @connecttext)

    'connect to the server
    eth.connect

  'Wait for connection to be established
  repeat while eth.con_est == false

  'wait for user to stop pushing button  
  repeat while gp.read & (NES_A | NES_B) <> $00
  
  'main game loop
  repeat
    'clear the display
    gr.clear
     
    'set the color for white and the pen for size 4
    'to draw the playing field
    gr.colorwidth(2,4)
     
    'draw the playing field
    gr.plot(55,0)
    gr.line(55,170)
     
    gr.plot(115,0)
    gr.line(115,170)
     
    gr.plot(0,55)
    gr.line(170,55)
     
    gr.plot(0,115)
    gr.line(170,115)
     
    'Set the text scale, space, and justification
    gr.textmode(5, 5, 0, $5)
     
    'Set the X, O, and selector box color to rainbow and
    'the width of the pen to size 2
    gr.colorwidth(1,2)

    'init the position variables
    x := 1
    y := 1

    'draw the center selection box
    if(movefirst == true)
      drawbox(x,y)

    'init the gameover varaible
    gameover := 0

    'init marker array
    repeat i from 0 to 2
      repeat j from 0 to 2
        marker[i*3 + j] := 0

    repeat while gameover == 0

      'If state == move then it is our turn to move
      repeat while state == move
        pad := gp.read
        if(pad & $FF <> $00)
         
          if(pad & $F) 
            case pad & $F
              NES_RIGHT:
                if(x < 2)
                  x++
              NES_LEFT:
                if(x > 0)
                  x--
              NES_UP:
                if(y < 2)
                  y++
              NES_DOWN:
                if(y > 0)
                  y--
            drawbox(x,y)
          
          else
          
            'determine if this square has been taken
            if(marker[x*3 + y] == 0)

              'mark this square as taken
              marker[x*3 + y] := 1
              
              drawletter(xo,x,y)
               
              'erase box
              drawbox(3,0)
               
              'setup the transmit buffer
              buffer[0] := xo
              buffer[1] := x
              buffer[2] := y
               
              'trasmit the data
              eth.TX(@buffer, 4)
               
              'determine if the game is over
              if(IsGameOver <> 0)
                state := over
              else
                state := wait

          repeat while gp.read & $FF <> $00
       
      'Else we are waiting for our opponent to move
      if(state == wait)

          'block waiting for data to come
          eth.RX(@buffer, 4, true)

          'draw the opponents letter
          drawletter(buffer[0], buffer[1], buffer[2])

          'mark this square as taken 
          marker[buffer[1]*3 + buffer[2]] := 2

          'determine if the game is over
          if(IsGameOver <> 0)
            state := over

          else

            'init the position variables
            x := 1
            y := 1

            'draw the center selection box
            drawbox(x,y)

            'change state
            state := move

      if(state == over)
      
        'Display win, loss, tie
        gr.textmode(8, 8, 6, $5)
        gr.color(3)
        if(IsGameOver == 1)
          gr.text(85, 110, @win)
        elseif(IsGameOver == 2)
          gr.text(85, 110, @loss)
        else
          gr.text(85, 110, @tie)
        gr.textmode(5, 5, 0, $5)
        gr.color(1)

        'Wait for player to stop pushing buttons
        repeat while gp.read & $FF <> $00

        'Wait for the player to hit any button to continue
        repeat while gp.read & $FF == $00

        'Wait for player to stop pushing buttons
        repeat while gp.read & $FF <> $00
        
        gameover := 1
        if(movefirst == true)
          movefirst := false
          state := wait
        else
          movefirst := true
          state := move
             
          
PUB IsGameOver | i,j

  'Check to see if we win
  if(marker[0*3 + 0] == 1 and marker[0*3 + 1] == 1 and marker[0*3 + 2] == 1)
    return 1
  if(marker[1*3 + 0] == 1 and marker[1*3 + 1] == 1 and marker[1*3 + 2] == 1)
    return 1
  if(marker[2*3 + 0] == 1 and marker[2*3 + 1] == 1 and marker[2*3 + 2] == 1)
    return 1
  if(marker[0*3 + 0] == 1 and marker[1*3 + 0] == 1 and marker[2*3 + 0] == 1)
    return 1
  if(marker[0*3 + 1] == 1 and marker[1*3 + 1] == 1 and marker[2*3 + 1] == 1)
    return 1
  if(marker[0*3 + 2] == 1 and marker[1*3 + 2] == 1 and marker[2*3 + 2] == 1)
    return 1
  if(marker[0*3 + 0] == 1 and marker[1*3 + 1] == 1 and marker[2*3 + 2] == 1)
    return 1
  if(marker[2*3 + 0] == 1 and marker[1*3 + 1] == 1 and marker[0*3 + 2] == 1)
    return 1

  'Check to see if our opponent won
  if(marker[0*3 + 0] == 2 and marker[0*3 + 1] == 2 and marker[0*3 + 2] == 2)
    return 2
  if(marker[1*3 + 0] == 2 and marker[1*3 + 1] == 2 and marker[1*3 + 2] == 2)
    return 2
  if(marker[2*3 + 0] == 2 and marker[2*3 + 1] == 2 and marker[2*3 + 2] == 2)
    return 2
  if(marker[0*3 + 0] == 2 and marker[1*3 + 0] == 2 and marker[2*3 + 0] == 2)
    return 2
  if(marker[0*3 + 1] == 2 and marker[1*3 + 1] == 2 and marker[2*3 + 1] == 2)
    return 2
  if(marker[0*3 + 2] == 2 and marker[1*3 + 2] == 2 and marker[2*3 + 2] == 2)
    return 2
  if(marker[0*3 + 0] == 2 and marker[1*3 + 1] == 2 and marker[2*3 + 2] == 2)
    return 2
  if(marker[2*3 + 0] == 2 and marker[1*3 + 1] == 2 and marker[0*3 + 2] == 2)
    return 2

  'Check to see if there is a tie
  repeat i from 0 to 8
    if(marker[i] == 0)
      return 0
      
  return 3                

PUB drawletter (letter, x, y)

  'draw the text
  if(letter == 0)
    gr.text(27 + x*60, 25 + y*60, @Xvar)
  else
    gr.text(27 + x*60, 25 + y*60, @Ovar)  

PUB drawbox (x, y) | i,j

  'begin by clearing all boxes
  'set the color to black
  gr.color(0)
  
  'draw the selector boxes
  repeat i from 0 to 2
    repeat j from 0 to 2
      gr.plot(50*i + 10*i, 50*j + 10*j)
      gr.line(50 + 50*i + 10*i, 50*j + 10*j)
      gr.line(50 + 50*i + 10*i, 50 + 50*j + 10*j)
      gr.line(50*i + 10*i, 50 + 50*j + 10*j)
      gr.line(50*i + 10*i, 50*j + 10*j)

  'set the color back to rainbow
  gr.color(1)

  if(x > 2)
    return

  gr.plot(50*x + 10*x, 50*y + 10*y)
  gr.line(50 + 50*x + 10*x, 50*y + 10*y)
  gr.line(50 + 50*x + 10*x, 50 + 50*y + 10*y)
  gr.line(50*x + 10*x, 50 + 50*y + 10*y)
  gr.line(50*x + 10*x, 50*y + 10*y)

DAT

Xvar                    byte    "X",0
Ovar                    byte    "O",0
servertext              byte    "A - Server",0
clienttext              byte    "B - Client",0
ttt_text                byte    "Tic Tac Toe",0
listentext              byte    "Listening",0
connecttext             byte    "Connecting",0
win                     byte    "Win",0
loss                    byte    "Loss",0
tie                     byte    "Tie",0        

tvparams                long    0               'status
                        long    1               'enable
                        long    %011_0000       'pins
                        long    %0000           'mode
                        long    0               'screen
                        long    0               'colors
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    10              'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    60_000_000'_xinfreq<<4  'broadcast
                        long    0               'auralcog
                        