''*****************************************************
''* Web Server for the Hydra Video Game Console
''* Written by: Shane Avery
''*****************************************************

CON

        _clkmode = xtal1 + pll8x
        _xinfreq = 10_000_000 + 0000

        'Constants for UDP for TCP port
        UDP = 0
        TCP = 1

        'Define the port number
        PORT = 80

VAR
        byte  dip[4]
        byte  subnet[4]
        byte  ip[4]
        byte  gateway[4]
        byte  mac[8]         

OBJ

        term    : "tv_terminal_010.spin"     ' instantiate the terminal object
        eth     : "W5100_drv_011.spin"       ' instantiate the W5100 driver

PUB begin

  'start the tv terminal
  term.start

  'start the W5100 driver
  eth.start

  'destination IP is not used for TCP host but it needs to be
  'something so we will give it a dummy address for now
  dip[0] := 192
  dip[1] := 168
  dip[2] := 1
  dip[3] := 1

  'fill out the rest of the arrays needed for intialization
  'W5100 IP address
  ip[0] := 192
  ip[1] := 168
  ip[2] := 1
  ip[3] := 100

  'Gateway
  gateway[0] := 192
  gateway[1] := 168
  gateway[2] := 1
  gateway[3] := 1

  'Subnet
  subnet[0] := 255
  subnet[1] := 255
  subnet[2] := 255
  subnet[3] := 0

  'W5100 MAC address
  mac[0] := $00
  mac[1] := $70
  mac[2] := $6e
  mac[3] := $69
  mac[4] := $73
  mac[5] := $0a
  
  'print title string and menu options
  term.pstring(@title)

  'initialize the W5100
  eth.init(@gateway, @subnet, @ip, @mac)
  
  'open the socket
  eth.open(TCP, PORT, PORT, @dip)

  repeat
    'listen for a connection
    eth.listen

    'display message
    term.pstring(@list)

    'Wait for connection to be established
    repeat while eth.con_est == false

    'display connected message
    term.pstring(@conn)

    'wait for something come be received
    repeat while eth.read_rsr == 0

    'display got request message
    term.pstring(@got)
  
    'Transmit the html code
    eth.TX(@header, 352)

    'display message that we served the page
    term.pstring(@served)


DAT

title   byte  "Hydra Web Server",13,13,0
list    byte  "Listening",13,0
conn    byte  "Connected",13,0
got     byte  "Got request",13,0
served  byte  "Page served",13,0

header  byte  "HTTP/1.1 200 OK",$d,$a
length  byte  "Content-length: 285",$d,$a
type    byte  "Content-type: text/html",$d,$a,$d,$a,$d,$a
head    byte  "<html><head><title>Hydra Web Server</title></head>"
body    byte  "<body><div align=",$22,"center",$22,"><b><font size=",$22,"+2",$22,">"
mess    byte  "Served by the Hydra Web Server v1.0</font></b></div>"
pic     byte  "<br><p></p><div align=",$22,"center",$22,"><img src=",$22,"http://www.shaneavery.com/projects/hydra/hydra2.jpg",$22
end     byte  " alt=",$22,$22," border=",$22,"0",$22,"></div><br></body></html>",$d,$a,0
